@@ -471,3 +471,93 @@ formatR = function(guiToolkit = 'RGtk2') {
 "%InLiNe_IdEnTiFiEr%" <- function(x, y) {
     x
 }
+
+
+
+##' Modified versions of parse() and deparse().
+##'
+##' the source code is masked to preserve comments, then this function
+##' uses \code{\link[base]{parse}} to return the parsed but
+##' unevaluated expressions in a list.
+##'
+##' @param text the source code as a character string to be passed to
+##' \code{\link{tidy.source}}
+##' @param ... for \code{\link{parse.tidy}}: other arguments to be passed to
+##' \code{\link{tidy.source}}; for \code{\link{deparse.tidy}}:
+##' arguments to be passed to \code{\link[base]{deparse}}
+##' @return \code{\link{parse.tidy}} returns the unevaluated
+##' expressions; \code{\link{deparse.tidy}} returns the character
+##' strings
+##' @author Yihui Xie <\url{http://yihui.name}>
+##' @note These functions are mainly designed for the package
+##' \pkg{pgfSweave}; they may not be useful to general users.
+##' @seealso \code{\link[base]{parse}}, \code{\link[base]{deparse}},
+##' \code{\link{tidy.source}}
+##' @export
+##' @examples
+##' src = c("    # a single line of comments is preserved",
+##' '1+1', '  ', 'if(TRUE){',
+##' "x=1  # comments begin with at least 2 spaces!", '}else{',
+##' "x=2;print('Oh no... ask the right bracket to go away!')}",
+##' '1*3 # this comment will be dropped!',
+##' "2+2+2    # 'short comments'",
+##' "lm(y~x1+x2)  ### only 'single quotes' are allowed in comments",
+##' "1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1  ## comments after a long line")
+##'
+##' (expr = parse.tidy(src))
+##'
+##' parse.tidy(src, keep.blank.line = TRUE)
+##'
+##' cat(deparse.tidy(expr))
+##'
+##' deparse.tidy(expr, width.cutoff = 50)
+##'
+parse.tidy = function(text, ...) {
+    tidy.res = tidy.source(text = text, output = FALSE, ...)
+    base::parse(text = tidy.res$text.mask)
+}
+
+##' Modified versions of parse() and deparse()
+##'
+##' it uses \code{\link[base]{deparse}} to turn the unevaluated (and
+##' masked) expressions into character strings; the masks will be
+##' removed to restore the real source code. See \code{\link{unmask.source}}.
+##' @param expr the unevaluated expressions (ideally as results from
+##' \code{\link{parse.tidy}})
+##' @rdname parse.tidy
+##' @export
+deparse.tidy = function(expr, ...) {
+    unmask.source(paste(base::deparse(expr, ...), collapse = '\n'))
+}
+
+##' Format the R scripts under a directory.
+##'
+##' This function first look for all the R scripts under a directory
+##' (using the pattern \code{"\\\\.[RrSsQq]$"}), then uses
+##' \code{\link{tidy.source}} to tidy these scripts. The original
+##' scripts will be overwritten with formatted code. You may need to
+##' back up the original directory first if you do not fully
+##' understand the tricks \code{\link{tidy.source}} is using.
+##' @param path the directory
+##' @param recursive whether to recursively look for R scripts under \code{path}
+##' @param ... other arguments to be passed to \code{\link{tidy.source}}
+##' @return NULL
+##' @author Yihui Xie <\url{http://yihui.name}>
+##' @seealso \code{\link{tidy.source}}
+##' @export
+##' @examples
+##' library(formatR)
+##'
+##' if (interactive()) {
+##' path = tempdir()
+##' file.copy(system.file('demo', package = 'base'), path, recursive=TRUE)
+##' tidy.dir(path, recursive=TRUE)
+##' }
+##'
+tidy.dir = function(path = '.', recursive = FALSE, ...) {
+    flist = list.files(path, pattern = '\\.[RrSsQq]$', full.names = TRUE, recursive = recursive)
+    for (f in flist) {
+        message('tidying ', f)
+        tidy.source(f, file = f, ...)
+    }
+}
